@@ -24,6 +24,7 @@
 static bool tls_initialized = false;
 
 static int verify_depth;
+static int verify_error = X509_V_OK;
 static int do_dump = 0;
 static SSL_CTX *CTX = NULL;
 SSL *tls_conn = NULL;
@@ -42,7 +43,7 @@ static int tls_loglevel = 0;
 
 
 /*
-**  Taken from OpenSSL apps/lib/s_cb.c.
+**  Taken from OpenSSL apps/s_cb.c.
 **  Tim -- this seems to just be giving logging messages.
 */
 static void
@@ -204,13 +205,12 @@ tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength UNUSED)
     static DH *ffdhe8192 = NULL;
     int level = 2; /* Default security level. */
 
-    /* Security levels have been introduced in OpenSSL 1.1.0 and
-     * LibreSSL 3.6.0.
+    /* Security levels have been introduced in OpenSSL 1.1.0, and still
+     * not present in LibreSSL 3.5.2.
      * Well, as this part of code is no longer active for these versions,
      * only keep it for possible future re-use. */
 #        if OPENSSL_VERSION_NUMBER >= 0x010100000L \
-            || (defined(LIBRESSL_VERSION_NUMBER)   \
-                && LIBRESSL_VERSION_NUMBER > 0x030600000L)
+            && !defined(LIBRESSL_VERSION_NUMBER)
     level = SSL_get_security_level(s);
 #        endif
 
@@ -245,7 +245,7 @@ tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength UNUSED)
 
 
 /*
-**  Taken from OpenSSL apps/lib/s_cb.c.
+**  Taken from OpenSSL apps/s_cb.c.
 */
 static int
 verify_callback(int ok, X509_STORE_CTX *ctx)
@@ -275,25 +275,19 @@ verify_callback(int ok, X509_STORE_CTX *ctx)
         syslog(L_NOTICE, "verify error:num=%d:%s", err,
                X509_verify_cert_error_string(err));
 
-        if (verify_depth < 0 || verify_depth >= depth) {
-            /* Accept the certificate in error if its depth lies within the
-             * first verify_depth intermediate CA certificates, or if no
-             * verification was asked. */
+        if (verify_depth >= depth) {
             ok = 1;
+            verify_error = X509_V_OK;
         } else {
             ok = 0;
+            verify_error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
         }
     }
 
     switch (err) {
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-        if (err_cert != NULL) {
-            X509_NAME_oneline(X509_get_issuer_name(err_cert), buf,
-                              sizeof(buf));
-            syslog(L_NOTICE, "issuer= %s", buf);
-        } else {
-            syslog(L_NOTICE, "cert has no issuer");
-        }
+        X509_NAME_oneline(X509_get_issuer_name(err_cert), buf, sizeof(buf));
+        syslog(L_NOTICE, "issuer= %s", buf);
         break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
     case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
@@ -312,9 +306,10 @@ verify_callback(int ok, X509_STORE_CTX *ctx)
 
 
 /*
-**  Taken from OpenSSL crypto/bio/bio_dump.c, modified to save a lot of strcpy
+**  Taken from OpenSSL crypto/bio/b_dump.c, modified to save a lot of strcpy
 **  and strcat by Matti Aarnio.
 */
+#    define TRUNCATE
 #    define DUMP_WIDTH 16
 
 static int
@@ -326,8 +321,17 @@ tls_dump(const char *s, int len)
     int i;
     int j;
     int rows;
+    int trunc;
     unsigned char ch;
 
+    trunc = 0;
+
+
+#    ifdef TRUNCATE
+    for (; (len > 0) && ((s[len - 1] == ' ') || (s[len - 1] == '\0')); len--)
+        trunc++;
+#    endif
+
     rows = (len / DUMP_WIDTH);
     if ((rows * DUMP_WIDTH) < len)
         rows++;
@@ -336,7 +340,7 @@ tls_dump(const char *s, int len)
         buf[0] = '\0'; /* Start with empty string. */
         ss = buf;
 
-        snprintf(ss, sizeof(buf), "%04x ", (unsigned int) (i * DUMP_WIDTH));
+        snprintf(ss, sizeof(buf), "%04x ", i * DUMP_WIDTH);
         ss += strlen(ss);
         for (j = 0; j < DUMP_WIDTH; j++) {
             if (((i * DUMP_WIDTH) + j) >= len) {
@@ -368,6 +372,14 @@ tls_dump(const char *s, int len)
             syslog(L_NOTICE, "%s", buf);
         ret += strlen(buf);
     }
+#    ifdef TRUNCATE
+    if (trunc > 0) {
+        snprintf(buf, sizeof(buf), "%04x - <SPACES/NULS>\n", len + trunc);
+        if (tls_loglevel > 0)
+            syslog(L_NOTICE, "%s", buf);
+        ret += strlen(buf);
+    }
+#    endif
     return (ret);
 }
 
@@ -377,7 +389,7 @@ tls_dump(const char *s, int len)
 **  private key (in key_file) and the cert (in cert_file).
 **  Both files may be identical.
 **
-**  This function is taken from OpenSSL apps/lib/s_cb.c.
+**  This function is taken from OpenSSL apps/s_cb.c.
 */
 static int
 set_cert_stuff(SSL_CTX *ctx, char *cert_file, char *key_file)
@@ -747,10 +759,8 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     }
 
     verify_depth = verifydepth;
-    /* Options for OPT_VERIFY in OpenSSL apps/s_server.c. */
     if (askcert != 0)
         verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
-    /* Options for OPT_UPPER_V_VERIFY in OpenSSL apps/s_server.c. */
     if (requirecert)
         verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
                         | SSL_VERIFY_CLIENT_ONCE;
@@ -822,13 +832,15 @@ bio_dump_cb(BIO *bio, int cmd, const char *argp, int argi, long argl UNUSED,
         return (ret);
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-        syslog(L_NOTICE, "read from %p [%p] (%d bytes => %ld (0x%lX))",
-               (void *) bio, (void *) argp, argi, ret, (unsigned long) ret);
+        syslog(L_NOTICE, "read from %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+               (unsigned long) bio, (unsigned long) argp, argi, ret,
+               (unsigned long) ret);
         tls_dump(argp, (int) ret);
         return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-        syslog(L_NOTICE, "write to %p [%p] (%d bytes => %ld (0x%lX))",
-               (void *) bio, (void *) argp, argi, ret, (unsigned long) ret);
+        syslog(L_NOTICE, "write to %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+               (unsigned long) bio, (unsigned long) argp, argi, ret,
+               (unsigned long) ret);
         tls_dump(argp, (int) ret);
     }
     return (ret);
@@ -843,25 +855,29 @@ bio_dump_cb(BIO *bio, int cmd, const char *argp, size_t len, int argi UNUSED,
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
         if (ret > 0 && processed != NULL) {
-            syslog(L_NOTICE, "read from %p [%p] (%lu bytes => %lu (0x%lX))",
-                   (void *) bio, (void *) argp, (unsigned long) len,
-                   (unsigned long) *processed, (unsigned long) *processed);
+            syslog(
+                L_NOTICE, "read from %08lX [%08lX] (%lu bytes => %lu (0x%lX))",
+                (unsigned long) bio, (unsigned long) argp, (unsigned long) len,
+                (unsigned long) *processed, (unsigned long) *processed);
             tls_dump(argp, (int) *processed);
         } else {
-            syslog(L_NOTICE, "read from %p [%p] (%lu bytes => %d (0x%lX))",
-                   (void *) bio, (void *) argp, (unsigned long) len, ret,
-                   (unsigned long) ret);
+            syslog(L_NOTICE,
+                   "read from %08lX [%08lX] (%lu bytes => %d (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, ret, (unsigned long) ret);
         }
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
         if (ret > 0 && processed != NULL) {
-            syslog(L_NOTICE, "write to %p [%p] (%lu bytes => %lu (0x%lX))",
-                   (void *) bio, (void *) argp, (unsigned long) len,
-                   (unsigned long) *processed, (unsigned long) *processed);
+            syslog(
+                L_NOTICE, "write to %08lX [%08lX] (%lu bytes => %lu (0x%lX))",
+                (unsigned long) bio, (unsigned long) argp, (unsigned long) len,
+                (unsigned long) *processed, (unsigned long) *processed);
             tls_dump(argp, (int) *processed);
         } else {
-            syslog(L_NOTICE, "write to %p [%p] (%lu bytes => %d (0x%lX))",
-                   (void *) bio, (void *) argp, (unsigned long) len, ret,
-                   (unsigned long) ret);
+            syslog(L_NOTICE,
+                   "write to %08lX [%08lX] (%lu bytes => %d (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, ret, (unsigned long) ret);
         }
     }
     return (ret);
